#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from openstack_dashboard.test.integration_tests.regions import tables

from sahara_dashboard.test.integration_tests.pages import basepage
from sahara_dashboard.test.integration_tests.pages import mixins
from sahara_dashboard.test.integration_tests.regions import forms


class ClusterTemplateForm(forms.TabbedFormRegion):
    CREATE_FIELD_MAPPING = (
        ('cluster_template_name', 'description', 'use_autoconfig',
            'anti_affinity'),
        (),
        ('CONF:general:Timeout for disk preparing',
            'CONF:general:Enable NTP service',
            'CONF:general:URL of NTP server',
            'CONF:general:Heat Wait Condition timeout',
            'CONF:general:Enable XFS')
    )

    def __init__(self, *args, **kwargs):
        kwargs['field_mappings'] = self.CREATE_FIELD_MAPPING
        super(ClusterTemplateForm, self).__init__(*args, **kwargs)

    def set_nodegroup_templates(self, node_group_templates):
        self.switch_to(1)
        template_el = self.src_elem.find_element_by_name('template')
        add_button = self.src_elem.find_element_by_id('add_group_button')
        to_add_templates = list(node_group_templates)
        groups_table = self.src_elem.find_element_by_id('groups_table')
        if groups_table.is_displayed():
            for row in groups_table.find_elements_by_css_selector(
                'tr.data-template-row'
            ):
                name = row.find_element_by_xpath('.//input[1]').get_attribute(
                    'value')
                if name in to_add_templates:
                    to_add_templates.remove(name)
                else:
                    row.find_element_by_css_selector(
                        'input[type=button]').click()

        for template_name in to_add_templates:
            for option in template_el.find_elements_by_tag_name('option'):
                if option.text == template_name:
                    option.click()
                    add_button.click()
                    break


class UpdateMixin(object):
    @tables.bind_row_action('edit')
    def get_edit_form(self, button, row):
        button.click()
        return ClusterTemplateForm(self.driver, self.conf)


class ClustertemplatesPage(mixins.PluginSelectMixin, mixins.DeleteMixin,
                           basepage.BaseDataProcessingPage):

    TABLE_NAME = "cluster_templates"

    @classmethod
    def get_table_mixins(cls):
        return (super(ClustertemplatesPage, cls).get_table_mixins() +
                (UpdateMixin,))

    @property
    def create_form(self):
        return ClusterTemplateForm(self.driver, self.conf)

    def create(self, plugin_name, plugin_version, name, node_group_templates,
               **kwargs):
        self.choose_plugin(plugin_name, plugin_version)
        form = self.create_form
        kwargs.update({
            'cluster_template_name': name
        })
        form.set_values(**kwargs)
        form.set_nodegroup_templates(node_group_templates)
        form.submit()

    def update(self, name, node_group_templates=(), **kwargs):
        row = self._get_row_with_name(name)
        form = self.table.get_edit_form(row)
        form.set_values(**kwargs)
        form.set_nodegroup_templates(node_group_templates)
        form.submit()

    def get_details(self, name):
        details = {'node_groups': {}}
        row = self._get_row_with_name(name)
        for node_group in row.cells['node_groups'].text.split('\n'):
            key, delimiter, value = node_group.partition(':')
            if delimiter == ':':
                details['node_groups'][key] = value.strip()
        self.table.src_elem.find_element_by_link_text(name).click()
        items = self.driver.find_elements_by_css_selector('div.detail dt')
        for item in items:
            key = item.text
            value_elem = item.find_element_by_xpath('./following-sibling::*')
            if value_elem.tag_name != "dd":
                continue
            value = value_elem.text
            details[key] = value
        self.driver.find_element_by_link_text('Configuration Details').click()
        items = self.driver.find_elements_by_css_selector('div.detail > dl li')
        for item in items:
            key, delimiter, value = item.text.partition(':')
            if delimiter == ':':
                details[key] = value.strip()
        for key, value in details.items():
            if '\n' in value:
                details[key] = set(value.split('\n'))
        return details

    def get_nodegroup_details(self, cluster_tpl_name, node_group_name):
        self.table.src_elem.find_element_by_link_text(cluster_tpl_name).click()
        self.driver.find_element_by_link_text('Node Groups').click()
        details = {}
        groups = self.driver.find_elements_by_css_selector(
            'div.tab-pane.active dl')
        for group in groups:
            if node_group_name not in group.text:
                continue
            for item in group.find_elements_by_tag_name('dt'):
                key = item.text
                value_elem = item.find_element_by_xpath(
                    './following-sibling::*')
                if value_elem.tag_name != "dd":
                    continue
                value = value_elem.text
                details[key] = value
        for key, value in details.items():
            if '\n' in value:
                details[key] = set(value.split('\n'))
        return details



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from django.core.urlresolvers import reverse

from sahara_dashboard.test import helpers as test


JOB_GUIDE_URL = reverse(
    'horizon:project:data_processing.jobs:jobex_guide')
JOB_GUIDE_RESET_URL = reverse(
    'horizon:project:data_processing.jobs:reset_jobex_guide',
    kwargs={"reset_jobex_guide": "true"})


class DataProcessingClusterGuideTests(test.TestCase):
    def test_jobex_guide(self):
        res = self.client.get(JOB_GUIDE_URL)
        self.assertTemplateUsed(
            res, 'job_wizard/jobex_guide.html')
        self.assertContains(res, 'Guided Job Execution')

    def test_jobex_guide_reset(self):
        res = self.client.get(JOB_GUIDE_RESET_URL)
        self.assertRedirectsNoFollow(res, JOB_GUIDE_URL)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from django.utils.translation import ugettext_lazy as _

from horizon import exceptions
from horizon import tabs

from sahara_dashboard.api import sahara as saharaclient
from sahara_dashboard.content.data_processing.jobs.data_sources \
    import tables as data_source_tables
from sahara_dashboard.content.data_processing \
    import tabs as sahara_tabs

LOG = logging.getLogger(__name__)


class DataSourcesTab(sahara_tabs.SaharaTableTab):
    table_classes = (data_source_tables.DataSourcesTable, )
    name = _("Data Sources")
    slug = "data_sources_tab"
    template_name = "horizon/common/_detail_table.html"

    def get_data_sources_data(self):
        try:
            data_sources = saharaclient.data_source_list(self.request)
        except Exception:
            data_sources = []
            exceptions.handle(self.request,
                              _("Unable to fetch data source list"))
        return data_sources


class GeneralTab(tabs.Tab):
    name = _("General Info")
    slug = "data_source_details_tab"
    template_name = "data_sources/_details.html"

    def get_context_data(self, request):
        data_source_id = self.tab_group.kwargs['data_source_id']
        try:
            data_source = saharaclient.data_source_get(request, data_source_id)
        except Exception as e:
            data_source = {}
            LOG.error("Unable to fetch data source details: %s" % str(e))

        return {"data_source": data_source}


class DataSourceDetailsTabs(tabs.TabGroup):
    slug = "data_source_details"
    tabs = (GeneralTab,)
    sticky = True



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.template import loader
from django.utils.translation import ugettext_lazy as _

from horizon import tables

from sahara_dashboard.content.data_processing.utils \
    import workflow_helpers as w_helpers


class UpdatePluginAction(tables.LinkAction):
    name = "update_plugin"
    verbose_name = _("Update Plugin")
    url = "horizon:project:data_processing.data_plugins:update"
    classes = ("ajax-modal", "btn-edit")


def versions_to_string(plugin):
    template_name = 'data_plugins/_list_versions.html'
    versions = w_helpers.get_enabled_versions(plugin)
    context = {"versions": versions}
    return loader.render_to_string(template_name, context)


class PluginsTable(tables.DataTable):
    title = tables.Column("title",
                          verbose_name=_("Title"),
                          link=("horizon:project:data_processing."
                                "data_plugins:plugin-details"))

    versions = tables.Column(versions_to_string,
                             verbose_name=_("Enabled Versions"))

    description = tables.Column("description",
                                verbose_name=_("Description"))

    class Meta(object):
        name = "plugins"
        verbose_name = _("Plugins")

        row_actions = (UpdatePluginAction,)
